package com.ruoyi.web.controller.qdtl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.qdtl.domain.MonitorStatic;
import com.ruoyi.qdtl.domain.TlArea;
import com.ruoyi.qdtl.domain.TlInspectionDeviceTrail;
import com.ruoyi.qdtl.domain.TlInspectionLocation;
import com.ruoyi.qdtl.domain.TlInspectionLocationLog;
import com.ruoyi.qdtl.domain.UserCheckPoint;
import com.ruoyi.qdtl.service.ITlAreaService;
import com.ruoyi.qdtl.service.ITlInspectionDeviceTrailService;
import com.ruoyi.qdtl.service.ITlInspectionLocationLogService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import com.ruoyi.qdtl.service.ITlMonitorService;
import com.ruoyi.qdtl.service.ITlPlanRecordService;
import com.ruoyi.system.domain.SysConfig;
import com.ruoyi.system.service.ISysConfigService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 监控设备管理Controller
 *
 * @author ruoyi
 * @date 2022-03-02
 */
@RestController
@RequestMapping("/qdtl/common")
@Slf4j
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TlCommonController extends BaseController {

    private final RedisCache redisCache;
    private final ISysConfigService sysConfigService;
    private final ITlAreaService tlAreaService;
    private final ITlInspectionLocationService tlInspectionLocationService;
    private final ITlInspectionDeviceTrailService tlInspectionDeviceTrailService;
    private final ITlInspectionLocationLogService tlInspectionLocationLogService;
    private final ITlMonitorService tlMonitorService;
    private final ITlPlanRecordService planRecordService;


    /**
     * 查询监控设备管理列表
     */
    @PostMapping("/http")
    public AjaxResult list(@RequestBody JSONObject jsonObject) {
        String url = jsonObject.getStr("url");
        String method = jsonObject.getStr("method");
        if (StrUtil.isBlank(method)) {
            method = "post";
        }
        String params = jsonObject.getStr("params");
        // 通过params解析缓存key
        String paramsPath = JSONUtil.parseObj(params).getStr("path");
        String result = "";
        if ("post".equals(method)) {
            try {
                result = HttpRequest.post(url).timeout(8000).body(params).execute().body();
                // 结果集存入缓存
                redisCache.setCacheObject("tl:http:" + paramsPath, result);
            } catch (Exception e) {
                log.error("http请求失败：{}", e.getMessage());
                // 从缓存中取值
                result = redisCache.getCacheObject("tl:http:" + paramsPath);
            }

        } else if ("get".equals(method)) {
            result = HttpUtil.get(url);
        }

        return AjaxResult.success("请求成功", result);
    }

    @GetMapping("/railwayLine")
    public AjaxResult railwayLine(@RequestParam(required = false) String key) {
        List<SysConfig> list = sysConfigService.selectConfigByLeftKey("tl.railway.");
        return AjaxResult.success(list);
    }

    /**
     * 地图上各类资源数据
     *
     * @return
     */
    @GetMapping("/resource")
    public AjaxResult resource(@RequestParam(required = false) String date) {
        Map<String, Object> map = new HashMap<>(8);
        // 区域
        List<TlArea> areas = tlAreaService.selectAreaList();
        map.put("area", areas);
        // 静态资源
        // 巡检点
        List<TlInspectionLocation> staticLocation = tlInspectionLocationService.selectLocationList();
        map.put("staticLocation", staticLocation);

        // 动态资源
        // 巡检轨迹
        if (StrUtil.isBlank(date)) {
            date = DateUtil.today();
        }
        List<TlInspectionDeviceTrail> deviceTrails = tlInspectionDeviceTrailService.selectListByDate(date);
        // 对轨迹按照设备编码分组
        Map<String, List<TlInspectionDeviceTrail>> deviceTrail = deviceTrails.stream().collect(Collectors.groupingBy(TlInspectionDeviceTrail::getDeviceCode));
        map.put("deviceTrail", deviceTrail);

        // 铁路沿线
        List<SysConfig> railway = sysConfigService.selectConfigByLeftKey("tl.railway.");
        map.put("railway", railway);
        return AjaxResult.success(map);
    }

    /**
     * 治理态势统计
     *
     * @param date
     * @return
     */
    @GetMapping("/govern")
    public AjaxResult govern(@RequestParam(required = false) String date) {
        Map<String, Object> map = new HashMap<>(8);
        if (StrUtil.isBlank(date)) {
            date = DateUtil.today();
        }
        // 巡检里程
        double xunTotalDistance = tlInspectionDeviceTrailService.queryDistance(date, "");
        map.put("xunTotalDistance", xunTotalDistance);
        // 巡检打卡数
        List<TlInspectionLocationLog> locationLogList = tlInspectionLocationLogService.queryByCondition(date, "", "");
        map.put("checkPointLogNum", locationLogList.size());
        // 视频资源统计
        List<MonitorStatic> cameraslist = tlMonitorService.queryMonitorStatic();
        map.put("cameraslist", cameraslist);
        // 巡检考核
        List<UserCheckPoint> checkStatic = tlInspectionLocationLogService.queryUserGroup();
        map.put("checkStatic", checkStatic);
        // 当日巡检完成率(总以及各个镇)
        Map<String, Object> checkPercent = planRecordService.queryCheckPercent(date);
        map.put("areaStatics", checkPercent.get("areaStatics"));
        map.put("totalStatic", checkPercent.get("totalStatic"));

        return AjaxResult.success(map);
    }
}
