package com.ruoyi.qdtl.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.qdtl.service.IQdCommonService;
import com.ruoyi.qdtl.service.ITlInspectionDeviceService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import com.ruoyi.qdtl.service.ITlPatrolmanService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Description: TODO
 * @Author: huangcheng
 * @Date: 2022/3/7
 * @Version V1.0
 */
@Service
@Slf4j
public class QdCommonServiceImpl implements IQdCommonService {

    @Value("${third.xuntoken}")
    private String xuntokenUrl;
    @Value("${third.xundevice}")
    private String xundeviceUrl;
    @Value("${third.xuncheckpointLog}")
    private String xuncheckpointLogUrl;
    @Value("${third.xuncheckpoint}")
    private String xuncheckpointUrl;
    @Value("${third.xunpatrolman}")
    private String xunpatrolmanUrl;
    @Value("${third.xuntrail}")
    private String xuntrailUrl;

    @Autowired
    private RedisCache redisCache;
    @Autowired
    private ITlInspectionLocationService tlInspectionLocationService;
    @Autowired
    private ITlInspectionDeviceService tlInspectionDeviceService;
    @Autowired
    private ITlPatrolmanService tlPatrolmanService;

    /**
     * 获取巡检接口的token
     *
     * @return
     */
    @Override
    public String getXunToken() {
        String token = redisCache.getCacheObject("xun:token");
        if (StrUtil.isNotBlank(token)) {
            return token;
        }
        Map<String, Object> params = new HashMap<>();
        params.put("username", "15190837633");
        params.put("password", "e10adc3949ba59abbe56e057f20f883e");
        String post = HttpUtil.post(xuntokenUrl, params);
        JSONObject postJson = JSONUtil.parseObj(post);
        if (200 == postJson.getInt("code")) {
            token = postJson.getStr("data");
            // 存入缓存。暂定为1小时过期
            redisCache.setCacheObject("xun:token", token, 1, TimeUnit.HOURS);
        } else {
            log.error("获取xun接口的token失败:{}", postJson);
            throw new ServiceException("获取xun接口的token失败");
        }
        return token;
    }

    /**
     * 设备列表
     *
     * @return
     */
    @Override
    public JSONArray getDevice() {
        String get = HttpRequest.get(xundeviceUrl).header("Authorization", this.getXunToken()).execute().body();
        JSONObject getJson = JSONUtil.parseObj(get);
        if (200 == getJson.getInt("code")) {
            JSONArray data = getJson.getJSONArray("data");
            return data;
        }
        log.error("调用我爱巡接口报错：{}", getJson);
        return new JSONArray();
    }

    /**
     * 查询巡检记录
     *
     * @param startTime 时间戳（秒）
     * @param endTime
     * @return
     */
    @Override
    @SneakyThrows
    public JSONArray getCheckpointLog(long startTime, long endTime) {
        if (ObjectUtil.isNull(endTime)) {
            endTime = DateUtil.current();
        }
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("startTime", startTime);
        paramMap.put("endTime", endTime);
        paramMap.put("pageSize", 5000);
        log.info("查询巡检记录--->请求参数：{}", paramMap);
        String get = HttpRequest.get(xuncheckpointLogUrl).form(paramMap).header("Authorization", this.getXunToken()).execute().body();
        JSONObject getJson = JSONUtil.parseObj(get);
        if (200 == getJson.getInt("code")) {
            JSONObject data = getJson.getJSONObject("data");
            if (data != null) {
                return data.getJSONArray("rows");
            }
        }
        log.error("调用我爱巡接口报错：{}", getJson);
        return null;
    }

    /**
     * 点位列表
     *
     * @return
     */
    @Override
    public JSONArray getCheckpoint() {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("pageSize", 5000);
        String get = HttpRequest.get(xuncheckpointUrl).form(paramMap).header("Authorization", this.getXunToken()).execute().body();
        JSONObject getJson = JSONUtil.parseObj(get);
        if (200 == getJson.getInt("code")) {
            JSONObject data = getJson.getJSONObject("data");
            if (data != null) {
                return data.getJSONArray("rows");
            }
        }
        log.error("调用我爱巡接口报错：{}", getJson);
        return null;
    }

    @Override
    public JSONArray getPatrolman() {
        String get = HttpRequest.get(xunpatrolmanUrl).header("Authorization", this.getXunToken()).execute().body();
        JSONObject getJson = JSONUtil.parseObj(get);
        if (200 == getJson.getInt("code")) {
            return getJson.getJSONArray("data");
        }
        log.error("调用我爱巡接口报错：{}", getJson);
        return null;
    }

    /**
     * 查询设备运行轨迹
     *
     * @param startTime
     * @param endTime
     * @param deviceId
     * @return
     */
    @Override
    public JSONArray getTrail(long startTime, long endTime, String deviceId) {
        if (ObjectUtil.isNull(endTime)) {
            endTime = DateUtil.current();
        }
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("startTime", startTime);
        paramMap.put("endTime", endTime);
        paramMap.put("deviceId", deviceId);
        String get = HttpRequest.get(xuntrailUrl).form(paramMap).header("Authorization", this.getXunToken()).execute().body();
        return JSONUtil.parseArray(get);
    }

    @Override
    public int getVersion(String key) {
        boolean hasKey = redisCache.hasKey(key);
        if (!hasKey) {
            // 查询数据库中最大的
            int maxVsersion = 0;
            if (key.equals("tl:version:checkpoint")) {
                maxVsersion = tlInspectionLocationService.queryMaxVsersion();
            } else if (key.equals("tl:version:devicce")) {
                maxVsersion = tlInspectionDeviceService.queryMaxVsersion();
            } else if (key.equals("tl:version:patrolman")) {
                maxVsersion = tlPatrolmanService.queryMaxVsersion();
            }
            redisCache.setCacheObject(key, maxVsersion);
        }
        return Convert.toInt(redisCache.incr(key, 1));
    }
}
