package com.ruoyi.qdtl.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.qdtl.domain.TlInspectionDeviceTrail;
import com.ruoyi.qdtl.mapper.TlInspectionDeviceTrailMapper;
import com.ruoyi.qdtl.service.ITlInspectionDeviceTrailService;
import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GeodeticCalculator;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 设备轨迹Service业务层处理
 *
 * @author ruoyi
 * @date 2022-03-12
 */
@Service
public class TlInspectionDeviceTrailServiceImpl implements ITlInspectionDeviceTrailService {
    @Autowired
    private TlInspectionDeviceTrailMapper tlInspectionDeviceTrailMapper;

    /**
     * 查询设备轨迹
     *
     * @param id 设备轨迹主键
     * @return 设备轨迹
     */
    @Override
    public TlInspectionDeviceTrail selectTlInspectionDeviceTrailById(Long id) {
        return tlInspectionDeviceTrailMapper.selectTlInspectionDeviceTrailById(id);
    }

    /**
     * 查询设备轨迹列表
     *
     * @param tlInspectionDeviceTrail 设备轨迹
     * @return 设备轨迹
     */
    @Override
    public List<TlInspectionDeviceTrail> selectTlInspectionDeviceTrailList(TlInspectionDeviceTrail tlInspectionDeviceTrail) {
        return tlInspectionDeviceTrailMapper.selectTlInspectionDeviceTrailList(tlInspectionDeviceTrail);
    }

    /**
     * 新增设备轨迹
     *
     * @param tlInspectionDeviceTrail 设备轨迹
     * @return 结果
     */
    @Override
    public int insertTlInspectionDeviceTrail(TlInspectionDeviceTrail tlInspectionDeviceTrail) {
        if (ObjectUtil.isNull(tlInspectionDeviceTrail.getCreateTime())) {
            tlInspectionDeviceTrail.setCreateTime(DateUtils.getNowDate());
        }
        return tlInspectionDeviceTrailMapper.insertTlInspectionDeviceTrail(tlInspectionDeviceTrail);
    }

    /**
     * 修改设备轨迹
     *
     * @param tlInspectionDeviceTrail 设备轨迹
     * @return 结果
     */
    @Override
    public int updateTlInspectionDeviceTrail(TlInspectionDeviceTrail tlInspectionDeviceTrail) {
        return tlInspectionDeviceTrailMapper.updateTlInspectionDeviceTrail(tlInspectionDeviceTrail);
    }

    /**
     * 批量删除设备轨迹
     *
     * @param ids 需要删除的设备轨迹主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionDeviceTrailByIds(Long[] ids) {
        return tlInspectionDeviceTrailMapper.deleteTlInspectionDeviceTrailByIds(ids);
    }

    /**
     * 删除设备轨迹信息
     *
     * @param id 设备轨迹主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionDeviceTrailById(Long id) {
        return tlInspectionDeviceTrailMapper.deleteTlInspectionDeviceTrailById(id);
    }

    @Override
    public List<TlInspectionDeviceTrail> selectListByDate(String date) {
        return tlInspectionDeviceTrailMapper.selectListByDate(date);
    }

    @Override
    public double queryDistance(String date, String deviceId) {
        List<TlInspectionDeviceTrail> deviceTrails = this.selectListByDate(date);
        // 对轨迹按照设备编码分组
        Map<String, List<TlInspectionDeviceTrail>> deviceTrail = deviceTrails.stream().collect(Collectors.groupingBy(TlInspectionDeviceTrail::getDeviceCode));
        // 对每一个设备循环累计里程
        double distance = 0;
        for (Map.Entry<String, List<TlInspectionDeviceTrail>> stringListEntry : deviceTrail.entrySet()) {
            List<TlInspectionDeviceTrail> origin = stringListEntry.getValue();
            if (origin.size() < 2) {
                continue;
            }
            // 对原始list分成2组list，一个去掉头节点，一个去掉尾节点
            List<TlInspectionDeviceTrail> subPre = CollUtil.sub(origin, 0, origin.size() - 1);
            List<TlInspectionDeviceTrail> subAfter = CollUtil.sub(origin, 1, origin.size());
            for (int i = 0; i < subPre.size(); i++) {
                String fencePre = subPre.get(i).getFence();
                String fenceAfter = subAfter.get(i).getFence();
                // 计算2点之间距离
                GlobalCoordinates source = new GlobalCoordinates(Convert.toDouble(fencePre.split(",")[1]), Convert.toDouble(fencePre.split(",")[0]));
                GlobalCoordinates target = new GlobalCoordinates(Convert.toDouble(fenceAfter.split(",")[1]), Convert.toDouble(fenceAfter.split(",")[0]));
                // 创建GeodeticCalculator,调用计算方法
                double ellipsoidalDistance = new GeodeticCalculator().calculateGeodeticCurve(Ellipsoid.WGS84, source, target).getEllipsoidalDistance();
                distance = NumberUtil.add(distance, ellipsoidalDistance);
            }
        }
        return NumberUtil.div(distance, 1000, 3);
    }
    
}
